##############################################################################
# Cognitive Diagnostic Models using R packages
# Code Author: Jorge L. Bazan.
##############################################################################


#1. Preliminaries 
rm(list = ls())
set.seed(1234)

#1. Reading data set
# Definition of folder of work
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) #For R studio

#Defining Q Matrix	
qq <- c(1,1, 
        1,1,
        1,0,
        1,1,
        1,0,
        1,0,
        1,0,
        1,0,
        1,1,
        1,1,
        0,1,
        1,1,
        1,1,
        1,1,
        1,1,
        0,1,
        0,1,
        1,1,
        0,1,
        1,1,
        0,1)
Q <- matrix(qq, ncol=2, byrow=TRUE) # Q-Matrix for our data. Q-Matrix must be defined acordingly to each dataset

#Reading data
bdi=read.csv("BDIdata.csv")

yy=bdi[,4:24]
Y=as.matrix(yy) #Matrix containing answers, with respondents in each line and items in each column
#edit(Y)

N <- nrow(Y) #Number of respondents
K <- ncol(Q) #Number of skills
J <- nrow(Q) #Number of items
C <- 2^K #Number of classes

##Visualization of profiles  
#delta0 <- rep(1,2^K)
As <- rep(0,K)
for(j in 1:K){
  temp <- combn(1:K,m=j)
  tempmat <- matrix(0,ncol(temp),K)
  for(j in 1:ncol(temp))
    tempmat[j,temp[,j]] <- 1
  As <- rbind(As,tempmat)
}
As <- as.matrix(As)  
As
  
  
#Running DINA model in dina package
###################################

#install dina package
if(!require(dina)) 
  install.packages("dina") 
library(dina)
chainLength= 10000
outchain<-dina(Y, Q, chain_length = chainLength)

burnin = 4000
## Summarize posterior samples for g and 1-s ----
chain_samples = burnin:chainLength
mGs = apply(outchain$GamS[, chain_samples], 1, mean)
sGs = apply(outchain$GamS[, chain_samples], 1, sd)


m1mSS = 1 - apply(outchain$SigS[, chain_samples], 1, mean)
s1mSS = apply(outchain$SigS[, chain_samples], 1, sd)


output = cbind(mGs, sGs, m1mSS, s1mSS)
colnames(output) = c('g Est', 'g SE', '1-s Est', '1-s SE')
rownames(output) = paste('Item', 1:J)
print(output, digits = 3)


## Summarize marginal skill distribution ----
# Via posterior samples for latent class proportions
PIoutput = cbind(apply(outchain$PIs, 1, mean), apply(outchain$PIs, 1, sd))
colnames(PIoutput) = c('EST', 'SE')
rownames(PIoutput) = apply(As, 1, paste0, collapse='')
print(PIoutput, digits = 3)

#Running DINA model in CDM package
###################################
if(!require(CDM)) 
  install.packages("CDM")  
library(CDM)

# estimate DINA model
outcdm <- din(Y, Q)
outcdm 
summary(outcdm)

# parameters and standard errors of DINA model
outcdm$item
#probabilities
outcdm$attribute.patt

#Running DINA model in GDINA
#############################
#install dina package
if(!require(GDINA)) 
  install.packages("GDINA") 
library(GDINA)

outgdina <- GDINA(dat = Y, Q = Q, model = "DINA")
outgdina

# summary information
summary(outgdina)

# structural parameters
# see ?coef
coef(outgdina, withSE = TRUE) # item probabilities of success & standard errors
coef(outgdina, what = "delta",withSE=TRUE) # delta parameters
coef(outgdina, what = "gs",withSE = TRUE) # guessing and slip parameters & standard errors
extract(outgdina,what = "catprob.parm")
coef(outgdina, what = "LCprob",withSE = TRUE)
coef(outgdina, what = "itemprob",withSE = TRUE)

extract(outgdina,"posterior.prob") #posterior probabilities (classification)
coef(outgdina, what = "lambda",withSE = TRUE)

# person parameters
# see ?personparm
personparm(outgdina) # EAP estimates of attribute profiles
personparm(outgdina, what = "MAP") # MAP estimates of attribute profiles
personparm(outgdina, what = "MLE") # MLE estimates of attribute profiles
personparm(outgdina, what = "mp") # MLE estimates of attribute profiles

#Running DINA model in JAGS package
#################################
#install R2jags package
if(!require(R2jags)) 
  install.packages("R2jags") 
library(R2jags)

DINA <- function(){
  for (i in 1:N) {
    for (j in 1:J) {
      for (k in 1:K) {w[i, j, k] <- pow(alpha[i, k], Q[j, k])
      }
      eta[i, j] <- prod(w[i, j, 1:K])
      p[i, j] <- pow((1 - s[j]), eta[i, j])*pow(g[j],(1 -eta[i, j]))
      Y[i, j] ~ dbern(p[i, j])}
    for (k in 1:K) {
      alpha[i, k] <- all.patterns[c[i], k]}
    c[i] ~ dcat(pai[1:C])
  }
  pai[1:C] ~ ddirch(delta[1:C])
  for (j in 1:J) {
    s[j] ~ dbeta(1, 1)
    g[j] ~ dbeta(1, 1)%_%T(0, 1 - s[j])}
  ##the posterior predictive model checking##
  for (i in 1:N){
    for (j in 1:J){
      teststat[i,j] <- pow(Y[i, j] - p[i, j], 2)/(p[i, j]*(1 - p[i, j]))
      Y_rep[i,j] ~ dbern(p[i,j])
      
      teststat_rep[i,j] <- pow(Y_rep[i, j] - p[i, j],2)/(p[i, j]*(1 - p[i, j]))
    }}
  teststatsum <- sum(teststat[1:N, 1:J])
  teststatsum_rep <- sum(teststat_rep[1:N, 1:J])
  ppp <- step(teststatsum_rep - teststatsum)
}


N = nrow(Y) 
J=nrow(Q)
K=ncol(Q)

gapp <- function(q){
  K <- ncol(q)
  q.entries <- as.list(1:K)
  for(k in 1:K){q.entries[[k]] <- sort(unique(c(0,q[,k])))}
  attr.patt <- as.matrix(expand.grid(q.entries))
}
all.patterns <- gapp(Q)
#View(all.patterns)
C=nrow(all.patterns)
delta=rep(1, C)


jags.data=list("N", "J", "K", "Y", "Q", "C", "all.patterns","delta")

#Step 4: Preliminary Study for Parameter Convergence
parameters <- c("s", "g", "pai")
#s: slipping parameter
#g: guessing parameter
#pai: posterior mixing proportion
jags.inits <- NULL #Initial values are not specified.

sim <- jags(data=jags.data, inits=jags.inits,
                parameters.to.save=parameters, 
                model.file=DINA, n.chains=2, n.iter=6000,
                n.burnin=2000,n.thin=5, DIC=TRUE)

sim
sim1 <- sim$BUGSoutput

E.itempar <- cbind(sim1$mean$g, sim1$mean$s, sim1$sd$g,
                   sim1$sd$s)
colnames(E.itempar)<-c("meang",'means','sdg','sds')
E.itempar

sim1$summary
sim1$DIC
sim1$mean$deviance
sim1$mean$ppp

 save.image("CodigoDina.RData")
 